#! /usr/bin/env python

"""PRCS Proxy.

Provide a simplified interface on PRCS files, locally or remotely.
The functionality is geared towards implementing some sort of
remote CVS like utility.  It is modeled after the similar module
FSProxy.

The module defines two classes:

PRCSProxyLocal  -- used for local access
PRCSProxyServer -- used on the server side of remote access

The corresponding client class, PRCSProxyClient, is defined in module
prcsclient.

The remote classes are instantiated with an IP address and an optional
verbosity flag.
"""

import server
import md5
import os
import fnmatch
import string
import tempfile
from prcslib import PRCS


class PRCSProxyLocal(PRCS):

    def __init__(self):
        PRCS.__init__(self)

    def __del__(self):
        PRCS.__del__(self)

    def get(self, project, rev, file, otherflags=''):
	rev = self.normalizerev(project, rev)
	if file[:-4] == project:
	    self.cd(self._TMPDIR)
	    self.checkout(project, rev, [file])
	    f = open(file)
	    data = f.read()
	    f.close()
	    self._remove(file)
	    self.back()
	else:
	    name_rev = self._namerev(project, rev, file)
	    f = self._open(name_rev, rflag = otherflags + ' -r')
	    data = f.read()
	    self._closepipe(f)
	return data

    def put(self, file, data):
	self.cd(self._TMPDIR)
        f = open(file, 'w')
        f.write(data)
        f.close()
	self.back()

class PRCSProxyServer(PRCSProxyLocal, server.SecureServer):

    def __init__(self, address, verbose = server.VERBOSE):
        PRCSProxyLocal.__init__(self)
        server.SecureServer.__init__(self, address, verbose)

    def _close(self):
        server.SecureServer._close(self)
        PRCSProxyLocal._close(self)

    def _serve(self):
        server.SecureServer._serve(self)
        # Retreat into start directory
        while self._dirstack: self.back()

def test_server():
    import string
    import sys
    if sys.argv[1:]:
        port = string.atoi(sys.argv[1])
    else:
        port = 4127
    proxy = PRCSProxyServer(('', port))
    proxy._serverloop()


def test():
    import sys
    if not sys.argv[1:] or sys.argv[1] and sys.argv[1][0] in '0123456789':
        test_server()
        sys.exit(0)
    proxy = PRCSProxyLocal()
    what = sys.argv[1]
    if hasattr(proxy, what):
        attr = getattr(proxy, what)
        if callable(attr):
            print apply(attr, tuple(sys.argv[2:]))
        else:
            print `attr`
    else:
        print "%s: no such attribute" % what
        sys.exit(2)


if __name__ == '__main__':
    test()
