#! /usr/bin/env python

"""WorkDir Proxy.

Provides a method for syncing two directory trees.

The module defines two classes:

WDProxyLocal  -- used for local access
WDProxyServer -- used on the server side of remote access

The corresponding client class, WDProxyClient, is defined in module
wdclient.

The remote classes are instantiated with an IP address and an optional
verbosity flag.
"""

import server
import md5
import os
import fnmatch
import string
import tempfile
import workdir


class WDProxyLocal(workdir.WorkDir):

    def __init__(self):
        workdir.WorkDir.__init__(self)
	env = os.environ
	dirfile = '.exportdir'
	if env.has_key('HOME'):
	    dirfile = os.path.join(env['HOME'], dirfile)
        self.dirs = {}
	self.dirs['.'] = os.path.abspath(os.curdir)
        f = open(dirfile, 'r')
        while 1:
           line = f.readline()
           if not line: break
           k, v = string.split(line)
           self.dirs[k] = v
	f.close()

    def __del__(self):
        workdir.WorkDir.__del__(self)

    def get(self, file):
        f = open(file)
        data = f.read()
        f.close()
        return data

    def put(self, file, data):
        f = open(file, 'w')
        f.write(data)
        f.close()

    def chwd(self, dir=None):
        if dir == None:
            return self.listwd()
        else:
	    # clean the stored database which are noe obsolete
	    self.dict = {}
            self.cd(self.dirs[dir])

    def listwd(self):
	return self.dirs


class WDProxyServer(WDProxyLocal, server.SecureServer):

    def __init__(self, address, verbose = server.VERBOSE):
        WDProxyLocal.__init__(self)
        server.SecureServer.__init__(self, address, verbose)

    def _close(self):
        server.SecureServer._close(self)
        WDProxyLocal._close(self)

    def _serve(self):
        server.SecureServer._serve(self)
        # Retreat into start directory
        # while self._dirstack: self.back()
        self._dirstack = []       


def test_server():
    import string
    import sys
    if sys.argv[1:]:
        port = string.atoi(sys.argv[1])
    else:
        port = 4128
    proxy = WDProxyServer(('', port))
    proxy._serverloop()


def test():
    import sys
    if not sys.argv[1:] or sys.argv[1] and sys.argv[1][0] in '0123456789':
        test_server()
        sys.exit(0)
    proxy = WDProxyLocal()
    what = sys.argv[1]
    if hasattr(proxy, what):
        attr = getattr(proxy, what)
        if callable(attr):
            print apply(attr, tuple(sys.argv[2:]))
        else:
            print `attr`
    else:
        print "%s: no such attribute" % what
        sys.exit(2)


if __name__ == '__main__':
    test()
